@@ -2,13 +2,573 @@ package XAO::testcases::FS::search;
 use strict;
 use XAO::Utils;
 use XAO::Objects;
+use Error qw(:try);
 
 use base qw(XAO::testcases::FS::base);
 
-##
+###############################################################################
+
+### This bug is known, there is no easy fix for it. And I am not sure it should be fixed at all.
+
+### sub test_mixed_levels {
+###     my $self=shift;
+###     my $odb=$self->get_odb();
+###
+###     $odb->fetch('/')->build_structure(
+###         Level0 => {
+###             type        => 'list',
+###             class       => 'Data::Level0',
+###             key         => 'l0_id',
+###             structure   => {
+###                 Level1 => {
+###                     type        => 'list',
+###                     class       => 'Data::Level1',
+###                     key         => 'l1_id',
+###                     structure   => {
+###                         text => {
+###                             type        => 'text',
+###                             maxlength   => 50,
+###                         },
+###                     },
+###                 },
+###                 text => {
+###                     type        => 'text',
+###                     maxlength   => 50,
+###                 },
+###             },
+###         },
+###     );
+###
+###     my ($l0_obj,$l1_obj);
+###     my ($l0_list,$l1_list);
+###     my $obj_id;
+###
+###     #1st elt has no 1st level list
+###     $l0_list=$odb->fetch('/Level0');
+###     $l0_obj=$l0_list->get_new();
+###     $l0_obj->put(text=>'foobar');
+###     $obj_id=$l0_list->put($l0_obj);
+### # If you uncomment this lines, search will be ok
+### # but with empty 1-st level list, search unable to find first record with text='foobar'
+### #    $l0_obj=$l0_list->get($obj_id);
+### #    my $l1_list=$l0_obj->get('Level1');
+### #    $l1_obj=$l1_list->get_new();
+### #    $l1_obj->put(text=>'zzz');
+### #    $l1_list->put($l1_obj);
+###
+###     #2nd elt has 1st level branch with 2 records ('foo','bar');
+###     $l0_obj=$l0_list->get_new();
+###     $l0_obj->put(text=>'something different');
+###     $obj_id=$l0_list->put($l0_obj);
+###     $l0_obj=$l0_list->get($obj_id);
+###     $l1_list=$l0_obj->get('Level1');
+###     $l1_obj=$l1_list->get_new();
+###     $l1_obj->put(text=>'foo');
+###     $l1_list->put($l1_obj);
+###     $l1_obj=$l1_list->get_new();
+###     $l1_obj->put(text=>'bar');
+###     $l1_list->put($l1_obj);
+###
+###     my $sr=$l0_list->search(
+###         [
+###             ['text','cs','bar'],
+###             'or',
+###             ['Level1/text','cs','bar'],
+###         ],
+###     );
+###
+###     my $nrows=scalar(@$sr);
+###     $self->assert($nrows==2,
+###         "Wrong search results in test_mixed_levels ($nrows instead of 2)");
+### }
+
+###############################################################################
+
+sub test_scan {
+    my $self=shift;
+
+    my $odb=$self->get_odb();
+
+    my $cust_list=$odb->fetch('/Customers');
+    my $cust_obj=$cust_list->get_new();
+
+    for(my $i=0; $i<100; ++$i) {
+        $cust_obj->put(name => sprintf('scancust%04u',$i));
+        $cust_list->put(sprintf('id%04u',$i) => $cust_obj);
+    }
+
+    $self->assert(scalar($cust_list->keys)==102,
+                  "Expected to have 102 customers in the list, got ".scalar($cust_list->keys));
+
+
+    my %tests=(
+        t01 => {
+            params  => {
+                block_size => 20,
+                search_query  => [
+                    'name','sw','scancust',
+                ],
+                search_options => {
+                    result  => [ 'customer_id','name' ],
+                    orderby => [ descend => 'customer_id' ],
+                    limit   => 90,  # overall limit
+                    offset  => 5,   # overall offset
+                },
+            },
+            expect => {
+                rows    => 90,
+                blocks  => 5,
+                first5  => 'id0094,id0093,id0092,id0091,id0090',
+            },
+        },
+        t02 => {
+            params  => {
+                block_size => 1,
+                search_query  => [
+                    'name','sw','scancust',
+                ],
+                search_options => {
+                    orderby => 'name',
+                },
+            },
+            expect => {
+                rows    => 100,
+                blocks  => 100,
+                first5  => 'id0000,id0001,id0002,id0003,id0004',
+            },
+        },
+        t03 => {
+            params  => {
+                block_size => 11,
+                search_options => {
+                    orderby => 'customer_id',
+                    offset  => 5,   # overall offset
+                },
+            },
+            expect => {
+                rows    => 97,
+                blocks  => 9,
+                first5  => 'id0003,id0004,id0005,id0006,id0007',
+            },
+        },
+        t04 => {
+            params  => {
+                block_size  => 40,
+                limit       => 35,  # overall offset
+                search_options => {
+                    orderby => 'customer_id',
+                    limit   => 50,  # low priority
+                },
+            },
+            expect => {
+                rows    => 35,
+                blocks  => 1,
+                first5  => 'c1,c2,id0000,id0001,id0002',
+            },
+        },
+    );
+
+    # Testing both collection and list scanning on the same set of tests.
+    #
+    my $cust_coll=$odb->collection(class => 'Data::Customer');
+
+    foreach my $cust_lc ($cust_list,$cust_coll) {
+        foreach my $testname (keys %tests) {
+            my $tdata=$tests{$testname};
+
+            my $called_before=0;
+            my $called_block=0;
+            my $called_row=0;
+            my $called_after=0;
+            my @srb;
+            my @srr;
+            $cust_lc->scan($tdata->{'params'},{
+                call_before => sub {
+                    my ($list,$args)=@_;
+                    ++$called_before;
+                    $self->assert(ref($args) && $args->{'block_size'},
+                                  "Expected to get back arguments in 'call_before'");
+                    $self->assert(ref($list),
+                                  "Expected to get back list in 'call_before'");
+                },
+                call_block => sub {
+                    my ($list,$args,$block)=@_;
+                    ++$called_block;
+                    ### dprint "call_block($called_block): ".Dumper($block);
+                    push(@srb,@$block);
+                    $self->assert((ref($args) && ref($args->{'call_block'})) ? 1 : 0,
+                                  "$testname: Expected to get back arguments in 'call_block'");
+                    $self->assert((ref($list)) ? 1 : 0,
+                                  "$testname: Expected to get back list in 'call_block'");
+                    $self->assert(ref($block) eq 'ARRAY',
+                                  "$testname: Expected to get results block in 'call_block'");
+                },
+                call_row => sub {
+                    my ($list,$args,$row)=@_;
+                    ++$called_row;
+                    push(@srr,$row);
+                    $self->assert((ref($args) && $args->{'search_options'}) ? 1 : 0,
+                                  "$testname: Expected to get back arguments in 'call_row'");
+                    $self->assert(ref($list),
+                                  "$testname: Expected to get back list in 'call_row'");
+                    $self->assert(defined($row),
+                                  "$testname: Expected to get a result row in 'call_row'");
+                },
+                call_after => sub {
+                    my ($list,$args)=@_;
+                    ++$called_after;
+                    $self->assert(ref($args) && $args->{'block_size'},
+                                  "$testname: Expected to get back arguments in 'call_after'");
+                    $self->assert(ref($list),
+                                  "$testname: Expected to get back list in 'call_after'");
+                },
+            });
+
+            my $expect=$tdata->{'expect'};
+
+            $self->assert($called_before==1,
+                          "$testname: Expected to have called_before==1, got $called_before");
+            $self->assert($called_block==$expect->{'blocks'},
+                          "$testname: Expected to have called_block==$expect->{'blocks'}, got $called_block");
+            $self->assert($called_row==$expect->{'rows'},
+                          "$testname: Expected to have called_row==$expect->{'rows'}, got $called_row");
+            $self->assert($called_after==1,
+                          "$testname: Expected to have called_after==1, got $called_after");
+
+            $self->assert(scalar(@srb)==scalar(@srr),
+                          "$testname: Expected to have the same data from call_block and call_row");
+            for(my $i=0; $i<@srr; ++$i) {
+                $self->assert($srb[$i] eq $srr[$i],
+                              "$testname: Expected to have the same data from call_block and call_row, got $srb[$i] and $srr[$i] at position $i");
+            }
+
+            # In the collection we're going to get collection IDs, not list
+            # IDs. Translating them to make the same tests work in both
+            # cases.
+            #
+            @srr=(map { ref($_) ? $_ : $cust_lc->get($_)->container_key } @srr);
+
+            my $first5=join(',',map { ref($_) ? $_->[0] : $_ } @srr[0..4] );
+
+            $self->assert($first5 eq $expect->{'first5'},
+                          "$testname: Expected first 5 elements to be $expect->{'first5'}, got $first5");
+        }
+    }
+}
+
+###############################################################################
+
+sub test_mixed_case {
+    my $self=shift;
+
+    my $odb=$self->get_odb();
+
+    my $custlist=$odb->fetch('/Customers');
+
+    my $customer=$custlist->get_new();
+
+    $self->assert(ref($customer),
+                  "Can't create Customer");
+
+    $customer->add_placeholder(
+        name        => 'MixedCase',
+        type        => 'text',
+        charset     => 'utf8',  # required for case-ign. collation
+        maxlength   => 100,
+        index       => 1,
+    );
+
+    $customer->add_placeholder(
+        name        => 'lowFirst',
+        type        => 'text',
+        charset     => 'latin1',
+        maxlength   => 1000,
+    );
+
+    $customer->put(
+        MixedCase       => 'mixed1',
+        lowFirst        => 'lc1',
+    );
+    $custlist->put(mct1 => $customer);
+    $customer->put(
+        MixedCase       => 'mixed2 with more',
+        lowFirst        => 'lc2',
+    );
+    $custlist->put(mct2 => $customer);
+
+    my $sr=$custlist->search('MixedCase','eq','mixed1');
+    $self->assert(@$sr==1,
+                  "Expected 1 result for mixed1");
+    $self->assert($sr->[0] eq 'mct1',
+                  "Expected to find 'mct1' for 'mixed1'");
+
+    $sr=$custlist->search('MixedCase','sw','mixed');
+    $self->assert(@$sr==2,
+                  "Expected 2 results for mixed1");
+
+    $sr=$custlist->search('lowFirst','eq','lc2');
+    $self->assert(@$sr==1,
+                  "Expected 1 result for lowFirst=lc2");
+    $self->assert($sr->[0] eq 'mct2',
+                  "Expected to find 'mct2' for 'lc2'");
+
+    $sr=$custlist->search(['lowFirst','eq','lc2'],'and',['MixedCase','eq','mixed1']);
+    $self->assert(@$sr==0,
+                  "Expected NO result for [lowFirst=lc2] and [MixedCase=mixed1]");
+}
+
+###############################################################################
+
+# Testing how returning multiple fields works -- 'result' option in search
+
+sub test_result_option {
+    my $self=shift;
+    my $odb=$self->get_odb();
+
+    my $cust_list=$odb->fetch('/Customers');
+
+    $cust_list->get_new->add_placeholder(
+        name        => 'desc',
+        type        => 'text',
+        maxlength   => 50,
+        charset     => 'utf8',
+    );
+    $cust_list->get_new->add_placeholder(
+        name        => 'common',
+        type        => 'text',
+        maxlength   => 50,
+        charset     => 'utf8',
+    );
+    $cust_list->get('c1')->put(name => 'name1', desc => 'aaaaa', common => 'common');
+    $cust_list->get('c2')->put(name => 'name2', desc => 'ddddd', common => 'common');
+
+    $cust_list->get_new->add_placeholder(
+        name        => 'Orders',
+        type        => 'list',
+        class       => 'Data::Order',
+        key         => 'order_id',
+    );
+    my $order_list=$cust_list->get('c1')->get('Orders');
+    my $order_obj=$order_list->get_new;
+    $order_obj->add_placeholder(
+        name        => 'total',
+        type        => 'real',
+    );
+    $order_obj->add_placeholder(
+        name        => 'text',
+        type        => 'text',
+        maxlength   => 100,
+    );
+    $order_obj->put(
+        total       => 123.45,
+        text        => 'c1o1',
+    );
+    $order_list->put('o1' => $order_obj);
+    $order_obj->put(
+        total       => 234.56,
+        text        => 'c1o2',
+    );
+    $order_list->put('o2' => $order_obj);
+    $order_obj->put(
+        total       => 345.67,
+        text        => 'c1o3',
+    );
+    $order_list->put('o3' => $order_obj);
+    $order_list=$cust_list->get('c2')->get('Orders');
+    $order_obj->put(
+        total       => 456.78,
+        text        => 'c2o1',
+    );
+    $order_list->put('o1' => $order_obj);
+    $order_obj->put(
+        total       => 567.89,
+        text        => 'c2o2',
+    );
+    $order_list->put('o2' => $order_obj);
+
+    my $order_coll=$odb->collection(class => 'Data::Order');
+
+    my %matrix=(
+        t01 => {
+            list    => $cust_list,
+            args    => [ 'desc','cs','d' ],
+            options => { result => [ qw(name desc) ], orderby => '-name', distinct => 'customer_id' },
+            result  => 'name2|ddddd',
+            rcount  => 2,
+        },
+        t02 => {
+            list    => $cust_list,
+            options => { result => [ qw(desc) ], orderby => 'name' },
+            result  => 'aaaaa;ddddd',
+            rcount  => 1,
+        },
+        t03 => {
+            list    => $cust_list,
+            args    => [ 'desc','cs','a' ],
+            options => { result => [ '#container_key','customer_id','name','desc' ], orderby => '-name' },
+            result  => 'c1|c1|name1|aaaaa',
+            rcount  => 4,
+        },
+        t04 => {
+            list    => $cust_list,
+            options => { result => [ '#collection_key' ], orderby => 'desc' },
+            result  => '1;2',         # this may break in other databases
+            rcount  => 1,
+        },
+        t05 => {
+            list    => $order_coll,
+            options => { result => [ '#id' ], orderby => 'text' },
+            result  => '1;2;3;4;5',     # this may break in other databases
+        },
+        t06 => {
+            list    => $cust_list,
+            options => { result => [ qw(common common) ], distinct => 'common' },
+            result  => 'common|common',
+        },
+        t07 => {
+            list    => $order_coll,
+            options => { result => [ '#connector', 'parent_unique_id' ] },
+            result  => sub {
+                my $row=shift;
+                $self->assert($row->[0] eq $row->[1],
+                    "Expected #connector value '$row->[0]' to equal parent_unique_id value '$row->[1]'");
+            },
+        },
+        t10 => {
+            list    => $cust_list,
+            args    => [ 'Orders/total','gt',300 ],
+            options => { result => [ qw(name common) ], orderby => 'customer_id' },
+            result  => 'name1|common;name2|common',
+            rcount  => 2,
+        },
+        t11 => {
+            list    => $cust_list,
+            args    => [ 'Orders/total','gt',300 ],
+            options => { result => [ qw(Orders/total Orders/text) ],
+                         orderby => [ ascend => 'customer_id', descend => 'Orders/total' ] },
+            result  => '345.67|c1o3;456.78|c2o1',
+            rcount  => 2,
+        },
+        t12 => {
+            list    => $cust_list,
+            options => { result => [ qw(Orders/total desc Orders/text) ],
+                         orderby => '-Orders/total' },
+            result  => '456.78|ddddd|c2o1;123.45|aaaaa|c1o1',
+            rcount  => 3,
+        },
+        t13 => {
+            list    => $cust_list,
+            options => { result => [ qw(Orders/total customer_id Orders/text) ],
+                         orderby => '-Orders/total' },
+            result  => '456.78|c2|c2o1;123.45|c1|c1o1',
+            rcount  => 3,
+        },
+        t14 => {
+            list    => $cust_list,
+            options => { result => [ qw(Orders/total customer_id Orders/text) ],
+                         orderby => '-Orders/total',
+                         distinct => 'Orders/text' },
+            result  => '567.89|c2|c2o2;456.78|c2|c2o1;345.67|c1|c1o3;234.56|c1|c1o2;123.45|c1|c1o1',
+            rcount  => 3,
+        },
+        t15 => {
+            list    => $cust_list,
+            options => { result => [ qw(Orders/total customer_id) ],
+                         orderby => 'Orders/total',
+                         distinct => 'Orders/text' },
+            result  => '123.45|c1;234.56|c1;345.67|c1;456.78|c2;567.89|c2',
+            rcount  => 2,
+        },
+        t20 => {
+            list    => $order_coll,
+            options => { result => [ 'text','../name','#container_key' ],
+                         orderby => '-total' },
+            result  => 'c2o2|name2|o2;c2o1|name2|o1;c1o3|name1|o3;c1o2|name1|o2;c1o1|name1|o1',
+            rcount  => 3,
+        },
+        t21 => {
+            list    => $cust_list,
+            options => { result => [ 'common' ] },
+            result  => 'common;common',
+            rcount  => 1,
+        },
+        t22 => {
+            list    => $cust_list,
+            args    => [ 'Orders/total', 'gt', 0 ],
+            options => { result => [ 'common' ] },
+            result  => 'common;common',
+            rcount  => 1,
+        },
+        t30 => {
+            list    => $order_list,
+            args    => [ [ 'total','gt',0 ], 'and', [ 'total','lt',1000 ] ],
+            options => { result => [ qw(order_id text total) ],
+                         orderby => 'text' },
+            result  => 'o1|c2o1|456.78;o2|c2o2|567.89',
+            rcount  => 3,
+        }
+    );
+
+    foreach my $t (sort keys %matrix) {
+        my $test=$matrix{$t};
+        my $list=$test->{'list'};
+
+        my $sr=$test->{'args'} ? $list->search($test->{'args'},$test->{'options'})
+                               : $list->search($test->{'options'});
+
+        ### dprint Dumper($sr);
+        $self->assert(ref($sr) eq 'ARRAY',
+                      "Test '$t', expected to get a list reference, got '".ref($sr)."'");
+
+        $self->assert(ref($sr->[0]) eq 'ARRAY',
+                      "Test '$t', expected to get a list of arrays, got '".ref($sr->[0])."'");
+
+        my $expect=$test->{'result'};
+
+        if(ref $expect eq 'CODE') {
+            foreach my $row (@$sr) {
+                $expect->($row);
+            }
+        }
+        else {
+            my $got=join(';',map {
+                join('|',$test->{'rcount'} ? @$_[0..($test->{'rcount'}-1)] : @$_);
+            } @$sr);
+
+            $self->assert($got eq $expect,
+                        "Test '$t', expected '$expect', got '$got'");
+        }
+    }
+}
+
+###############################################################################
+
+# reported by enn@, 2006/05/03
+
+sub test_empty_array_ref {
+    my $self=shift;
+    my $odb=$self->get_odb();
+    my $customers=$odb->fetch('/Customers');
+
+    my $got;
+    try {
+        my $sr=$customers->search([ 'name','sw', [ ] ]);
+        my $got=join(',',sort @$sr);
+    }
+    otherwise {
+        my $e=shift;
+        dprint "Expected error: $e";
+    };
+
+    $self->assert(!defined $got,
+                  "Bug in empty array reference treatment");
+}
+
+##################################################################################
+
 # Test for a bug in MySQL_DBI driver in handling on multi-value returns
 # in search.
-#
+
 sub test_bug_20030505 {
     my $self=shift;
     my $odb=$self->get_odb();
@@ -113,10 +673,12 @@ sub test_real_deep {
                                 name => {
                                     type        => 'text',
                                     maxlength   => 50,
+                                    charset     => 'latin1',
                                 },
                                 desc => {
                                     type        => 'text',
                                     maxlength   => 300,
+                                    charset     => 'utf8',
                                 },
                             },
                         },
@@ -391,7 +953,7 @@ sub test_real_deep {
                 'and',
                 [ 'C/desc', 'sw', 'qwerty' ],
                 { index => '/X/name',
-                  orderby => [ descend => '/X/A/B/name' ],
+                  orderby => '-/X/A/B/name',
                 }
             ],
             uri     => '/X/a217/A/a218/B',
@@ -434,6 +996,38 @@ sub test_real_deep {
             class   => 'Data::E',
             result  => '14,57,23,90,103,33,105,22,80,27',
         },
+        t6_2 => {
+            args    => [
+                [ '/project','cs','new' ],
+                { orderby   => 'name',
+                  limit     => 10,
+                  offset    => 0,
+                },
+            ],
+            class   => 'Data::E',
+            result  => '14,57,23,90,103,33,105,22,80,27',
+        },
+        t6_3 => {
+            args    => [
+                [ '/project','cs','new' ],
+                { orderby   => 'name',
+                  limit     => 10,
+                  offset    => 1,
+                },
+            ],
+            class   => 'Data::E',
+            result  => '57,23,90,103,33,105,22,80,27,74',
+        },
+        t6_4 => {
+            args    => [
+                [ '/project','cs','new' ],
+                { orderby   => 'name',
+                  offset    => 120,
+                },
+            ],
+            class   => 'Data::E',
+            result  => '30,19,73,65,92',
+        },
         t7 => {
             args    => [
                 [ 'B/*/C/*/name', 'sw', 'e' ],
@@ -513,6 +1107,8 @@ sub test_real_deep {
     }
 }
 
+###############################################################################
+
 sub test_search {
     my $self=shift;
 
@@ -525,14 +1121,20 @@ sub test_search {
     $self->assert(ref($customer),
                   "Can't create Customer");
 
-    $customer->add_placeholder(name => 'short',
-                               type => 'text',
-                               maxlength => 100,
-                               index => 1);
+    $customer->add_placeholder(
+        name        => 'short',
+        type        => 'text',
+        charset     => 'utf8',  # required for case-ign. collation
+        maxlength   => 100,
+        index       => 1,
+    );
 
-    $customer->add_placeholder(name => 'long',
-                               type => 'text',
-                               maxlength => 1000);
+    $customer->add_placeholder(
+        name        => 'long',
+        type        => 'text',
+        charset     => 'utf8',
+        maxlength   => 1000,
+    );
 
     ##
     # For deeper search
@@ -580,7 +1182,7 @@ lie undisturbed for years.  Then one night it suddenly hatches  discards its
 outer skin that crumbles into dust  and emerges as a totally unidentifiable
 little metal object with flanges at both ends and a sort of ridge and a hole
 for a screw.  This  when found  will get thrown away.  No one knows what the
-screwdriver is supposed to gain from this.  Nature  in her infinite wisdom 
+screwdriver is supposed to gain from this.  Nature  in her infinite wisdom
 is presumably working on it.
 EOT
 
@@ -821,7 +1423,6 @@ EOT
     $self->assert(scalar(keys %a) == 149,
                   "Non-unique ID in search results, test 14");
 
-
     ##
     # Cleaning up
     #
@@ -862,7 +1463,8 @@ sub test_multiple_branches {
             key         => 'order_id',
             structure   => {
                 name => {
-                    type    => 'text',
+                    type        => 'text',
+                    maxlength   => 100,
                 },
             },
         },
@@ -872,7 +1474,8 @@ sub test_multiple_branches {
             key         => 'product_id',
             structure   => {
                 name => {
-                    type    => 'text',
+                    type        => 'text',
+                    maxlength   => 100,
                 },
             },
         },
@@ -932,15 +1535,16 @@ sub test_multiple_branches {
     $ids=$customers->search([ 'Orders/name', 'eq', 'kaaau' ],
                             'or',
                             [ 'Products/name', 'eq', 'ru' ],
-                            { orderby => 'customer_id' });
+                            { orderby => '-customer_id' });
 
     $t_ids=join(",",@$ids);
-    $self->assert($t_ids eq 'c2,c3',
-                  "Wrong search results for multi-branch search (got '$t_ids', expect 'c2,c3')");
+    $self->assert($t_ids eq 'c3,c2',
+                  "Wrong search results for multi-branch search (got '$t_ids', expect 'c3,c2')");
 
     $ids=$customers->search([ 'Orders/name', 'eq', 'foo' ],
                             'and',
-                            [ 'Products/name', 'eq', 'bar' ]);
+                            [ 'Products/name', 'eq', 'bar' ],
+                            { orderby => 'customer_id' });
 
     $t_ids=join(",",@$ids);
     $self->assert($t_ids eq 'c1,screw',
@@ -1072,4 +1676,5 @@ sub deep_variants {
                   "Wrong complex deep search in test_deep_variants");
 }
 
+###############################################################################
 1;
